@@ -23,6 +23,7 @@
 
 #include <KDE/KCmdLineArgs>
 #include <KDE/KConfig>
+#include <KDE/KConfigGroup>
 #include <KDE/KGlobal>
 #include <kwindowsystem.h>
 #include <KDE/KLocale>
@@ -106,6 +107,9 @@ KWD::Decorator::Decorator (Display* display,
 
     mActiveId = 0;
 
+    KConfigGroup cfg (KSharedConfig::openConfig("plasmarc"), QString("Theme"));
+    Plasma::Theme::defaultTheme ()->setThemeName (cfg.readEntry ("name"));
+
     Atoms::init ();
 
     (void *) new KWinAdaptor (this);
@@ -237,9 +241,9 @@ KWD::Decorator::enableDecorations (Time timestamp,
 
     connect (&mIdleTimer, SIGNAL (timeout ()), SLOT (processDamage ()));
 
-    connect (Plasma::Theme::self (), SIGNAL (changed ()),
+    connect (Plasma::Theme::defaultTheme (), SIGNAL (themeChanged ()),
 	     SLOT (plasmaThemeChanged ()));
-    
+
     // select for client messages
     XSelectInput (QX11Info::display(), QX11Info::appRootWindow(),
 		  StructureNotifyMask | PropertyChangeMask);
@@ -599,8 +603,17 @@ KWD::Decorator::x11EventFilter (XEvent *xevent)
 		xbe2.y = xbe->y - child->pos ().y ();
 	    }
 
+	    client->setFakeRelease (false);
 	    QApplication::x11ProcessEvent ((XEvent *) &xbe2);
 
+	    /* We won't get a button release event, because of the screengrabs
+	       in compiz */
+	    if (client->getFakeRelease () && xevent->type == ButtonPress)
+	    {
+		xbe2.type = ButtonRelease;
+		QApplication::x11ProcessEvent ((XEvent *) &xbe2);
+	    }
+
 	    return true;
 	}
     } break;
@@ -631,15 +644,6 @@ KWD::Decorator::x11EventFilter (XEvent *xevent)
 		    client->showWindowMenu (pos);
 		}
 	    }
-	    else if (action == Atoms::toolkitActionMainMenuAtom)
-	    {
-	    }
-	    else if (action == Atoms::toolkitActionRunDialogAtom)
-	    {
-		QDBusInterface krunner ("org.kde.krunner", "/Interface",
-					"org.kde.krunner.Interface");
-		krunner.call ("display", "");
-	    }
 	    else if (action == Atoms::toolkitActionForceQuitDialogAtom)
 	    {
 		if (mClients.contains (xevent->xclient.window))
