@@ -1911,15 +1911,17 @@ KWD::Window::processDamage (void)
     {
 	r2.translate (-xOff, -yOff);
 
-	decor_blend_top_border_picture (qt_xdisplay (),
-					&mContext,
-					mPicture,
-					xOff, xOff,
-					mTexturePicture,
-					&mLayout,
-					r2.handle (),
-					(unsigned short) (alpha * 0xffff),
-					shade_alpha);
+	decor_blend_border_picture (qt_xdisplay (),
+				    &mContext,
+				    mPicture,
+				    xOff, xOff,
+				    mTexturePicture,
+				    &mLayout,
+				    BORDER_TOP,
+				    r2.handle (),
+				    (unsigned short) (alpha * 0xffff),
+				    shade_alpha,
+				    TRUE);
     }
 
     xOff = 0;
@@ -1932,15 +1934,17 @@ KWD::Window::processDamage (void)
     {
 	r2.translate (-xOff, -yOff);
 
-	decor_blend_bottom_border_picture (qt_xdisplay (),
-					   &mContext,
-					   mPicture,
-					   xOff, yOff,
-					   mTexturePicture,
-					   &mLayout,
-					   r2.handle (),
-					   (unsigned short) (alpha * 0xffff),
-					   shade_alpha);
+	decor_blend_border_picture (qt_xdisplay (),
+				    &mContext,
+				    mPicture,
+				    xOff, yOff,
+				    mTexturePicture,
+				    &mLayout,
+				    BORDER_BOTTOM,
+				    r2.handle (),
+				    (unsigned short) (alpha * 0xffff),
+				    shade_alpha,
+				    TRUE);
     }
 
     xOff = 0;
@@ -1953,15 +1957,17 @@ KWD::Window::processDamage (void)
     {
 	r2.translate (-xOff, -yOff);
 
-	decor_blend_left_border_picture (qt_xdisplay (),
-					 &mContext,
-					 mPicture,
-					 xOff, yOff,
-					 mTexturePicture,
-					 &mLayout,
-					 r2.handle (),
-					 (unsigned short) (alpha * 0xffff),
-					 shade_alpha);
+	decor_blend_border_picture (qt_xdisplay (),
+				    &mContext,
+				    mPicture,
+				    xOff, yOff,
+				    mTexturePicture,
+				    &mLayout,
+				    BORDER_LEFT,
+				    r2.handle (),
+				    (unsigned short) (alpha * 0xffff),
+				    shade_alpha,
+				    TRUE);
     }
 
     xOff = mContext.extents.left + mGeometry.width ();
@@ -1974,15 +1980,17 @@ KWD::Window::processDamage (void)
     {
 	r2.translate (-xOff, -yOff);
 
-	decor_blend_right_border_picture (qt_xdisplay (),
-					  &mContext,
-					  mPicture,
-					  xOff, yOff,
-					  mTexturePicture,
-					  &mLayout,
-					  r2.handle (),
-					  (unsigned short) (alpha * 0xffff),
-					  shade_alpha);
+	decor_blend_border_picture (qt_xdisplay (),
+				    &mContext,
+				    mPicture,
+				    xOff, yOff,
+				    mTexturePicture,
+				    &mLayout,
+				    BORDER_RIGHT,
+				    r2.handle (),
+				    (unsigned short) (alpha * 0xffff),
+				    shade_alpha,
+				    TRUE);
     }
 
     mDamage = QRegion ();
