@@ -122,7 +122,7 @@ annoCairoContext (CompScreen *s)
 
 	if (!bindPixmapToTexture (s, &as->texture, as->pixmap, w, h, 32))
 	{
-	    compLogMessage (s->display, "annotate", CompLogLevelError,
+	    compLogMessage ("annotate", CompLogLevelError,
 			    "Couldn't bind pixmap 0x%x to texture",
 			    (int) as->pixmap);
 
@@ -742,7 +742,7 @@ annoSetDisplayOption (CompPlugin      *plugin,
 
 static const CompMetadataOptionInfo annoDisplayOptionInfo[] = {
     { "initiate_button", "button", 0, annoInitiate, annoTerminate },
-    { "draw_button", "button", 0, annoDraw, 0 },
+    { "draw", "action", 0, annoDraw, 0 },
     { "erase_button", "button", 0, annoEraseInitiate, annoTerminate },
     { "clear_key", "key", 0, annoClear, 0 },
     { "clear_button", "button", 0, annoClear, 0 },
@@ -890,8 +890,9 @@ annoGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) annoGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
