@@ -339,7 +339,7 @@ resizeInitiate (CompDisplay     *d,
 	mask = getIntOptionNamed (option, nOption, "direction", 0);
 
 	/* Initiate the resize in the direction suggested by the
-	 * quarter of the window the mouse is in, eg drag in top left
+	 * sector of the window the mouse is in, eg drag in top left
 	 * will resize up and to the left.  Keyboard resize starts out
 	 * with the cursor in the middle of the window and then starts
 	 * resizing the edge corresponding to the next key press. */
@@ -349,11 +349,25 @@ resizeInitiate (CompDisplay     *d,
 	}
 	else if (!mask)
 	{
-	    mask |= ((x - w->serverX) < (w->serverWidth / 2)) ?
-		ResizeLeftMask : ResizeRightMask;
+	    unsigned int sectorSizeX = w->serverWidth / 3;
+	    unsigned int sectorSizeY = w->serverHeight / 3;
+	    unsigned int posX        = x - w->serverX;
+	    unsigned int posY        = y - w->serverY;
 
-	    mask |= ((y - w->serverY) < (w->serverHeight / 2)) ?
-		ResizeUpMask : ResizeDownMask;
+	    if (posX < sectorSizeX)
+		mask |= ResizeLeftMask;
+	    else if (posX > (2 * sectorSizeX))
+		mask |= ResizeRightMask;
+
+	    if (posY < sectorSizeY)
+		mask |= ResizeUpMask;
+	    else if (posY > (2 * sectorSizeY))
+		mask |= ResizeDownMask;
+
+	    /* if the pointer was in the middle of the window,
+	       do nothing */
+	    if (!mask)
+		return FALSE;
 	}
 
 	if (otherScreenGrabExist (w->screen, "resize", 0))
@@ -450,6 +464,10 @@ resizeInitiate (CompDisplay     *d,
 					   CompWindowGrabResizeMask |
 					   CompWindowGrabButtonMask);
 
+	    if (d->opt[COMP_DISPLAY_OPTION_RAISE_ON_CLICK].value.b)
+		updateWindowAttributes (w,
+					CompStackingUpdateModeAboveFullscreen);
+
 	    /* using the paint rectangle is enough here
 	       as we don't have any stretch yet */
 	    resizeGetPaintRectangle (d, &box);
@@ -924,6 +942,20 @@ resizeHandleEvent (CompDisplay *d,
 		    }
 		}
 	    }
+	    else if (rd->w && event->xclient.data.l[2] == WmMoveResizeCancel)
+	    {
+		if (rd->w->id == event->xclient.window)
+		{
+		    int option;
+
+		    option = RESIZE_DISPLAY_OPTION_INITIATE_BUTTON;
+		    resizeTerminate (d, &rd->opt[option].value.action,
+				     CompActionStateCancel, NULL, 0);
+		    option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
+		    resizeTerminate (d, &rd->opt[option].value.action,
+				     CompActionStateCancel, NULL, 0);
+		}
+	    }
 	}
 	break;
     case DestroyNotify:
@@ -1003,7 +1035,7 @@ resizePaintRectangle (CompScreen              *s,
     glPushMatrix ();
 
     transformToScreenSpace (s, output, -DEFAULT_Z_CAMERA, &sTransform);
-    
+
     glLoadMatrixf (sTransform.m);
 
     glDisableClientState (GL_TEXTURE_COORD_ARRAY);
@@ -1394,8 +1426,9 @@ resizeGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) resizeGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
