@@ -1751,8 +1751,9 @@ waterGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) waterGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
