@@ -1128,8 +1128,9 @@ zoomGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) zoomGetScreenOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
